/*
 * This is the class that serves as the feedback for how to react
 * between services and javascript.
 */

/*
Copyright (c) 2009, Dan "Ducky" Little & GeoMOOSE.org

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

window.GeoMOOSE = {
	splitPaths : function(layerPaths) {
		return (new String(layerPaths)).split(':');
	},
	/* Layer Functions */
	changeLayerVisibility : function(layerPath, visibility) {
		var paths = this.splitPaths(layerPath);
		for(var p = 0; p < paths.length; p++) {
			var e = document.getElementById('catalog-'+paths[p]);
			if(!e && !Catalog.setLayerVisibility(paths[p], visibility)) {
				OpenLayers.Console.log('Layer '+paths[p]+' does not exist');
			} else if(e && e.checked != visibility) {
				e.checked = visibility;
				e.onclick();
			}
		}
	},

	turnLayerOn : function(layerPaths) {
		this.changeLayerVisibility(layerPaths, true);
	},

	turnLayerOff : function(layerPaths) {
		this.changeLayerVisibility(layerPaths, false);
	},

	refreshLayers : function(layerNames) {
		if(!(layerNames instanceof Array)) {
			layerNames = [layerNames];
		}
		Catalog.refreshLayer(layerNames.join(':'));
	},

	changeLayerUrl : function(layerName, url) {
		Catalog.setLayerUrl(layerName, url);
	},

	updateLayerParameters : function(layerName, paramObject) {
		Catalog.setLayerParameters(layerName, paramObject);
	},

	getLayerUrl : function(layerName) {
		return Catalog.getLayerUrl(layerName);
	},

	getLayerParameters : function(layerName) {
		return Catalog.getLayerParameters(layerName);
	},

	getLayerOpacity : function(layerName) {
		return Catalog.getLayerOpacity(layerName)*100;
	},

	getVisibleLayers : function() {
		return Catalog.getVisibleLayers();
	},

	getExtent: function() {
		return Map.getExtent().toArray();
	},

	/* Start a service with some given parameters */
	startService : function(serviceName, settingsObj, forceStart) {
		Services.startService(serviceName, settingsObj, forceStart);
	},

	/* Change to a different tab in the interface */
	changeTab : function(tabName) {
		Tabs.showTab(tabName);
	},

	/* Zoom to a point and a buffer */
	zoomToPoint : function(x,y,buffer) {
		Map.zoomToExtent(OpenLayers.Bounds.fromArray([x-buffer, y-buffer, x+buffer, y+buffer]));
	},

	/* Zoom to an extent */
	zoomToExtent : function(minx,miny,maxx,maxy) {
		Map.zoomToExtent(OpenLayers.Bounds.fromArray([minx,miny,maxx,maxy]));
	},

	/* Popup Functions */
	addPopup : function(x,y,w,h,html) {
		addPopup(x,y,w,h,html);
	},

	clearPopups : function() {
		while(Map.popups[0]) {
			Map.removePopup(Map.popups[0]);
		}
	},

	/* GeoMOOSE Template Processing Utility */
	processTemplate : function (template, replace_dict) {
		var ret_string = new String(template);
		for(var d in replace_dict) {
			ret_string = ret_string.replace('%'+d+'%', replace_dict[d]);
		}
		return ret_string
	},

	bookmark: function() {
		document.location = geomooseBookmark();
	},

	register: function(event_type, ref_obj, func) {
		GM_Events.register(event_type, ref_obj, func);
	}
};

/*
 * GeoMOOSE User Extensions Library 
 */


GeoMOOSE.UX = {
	registered_extensions: {},

	/* 
	 * The names are used as keys.
	 */
	register: function(extension_name) {
		GeoMOOSE.UX.registered_extensions[extension_name] = eval(extension_name);
	},

	unregister: function(extension_name) {
		delete GeoMOOSE.UX.registered_extensions[extension_name];
	},

	/*
	 * Return the extension objects
	 */
	getExtensions: function() {
		var all_extensions = new Array();
		for(var k in GeoMOOSE.UX.registered_extensions) {
			all_extensions.push(GeoMOOSE.UX.registered_extensions[k]);
		}
		return all_extensions;
	}
};

/*
 * I made this a class to give User Extensions a prayer of surviving an upgrade.
 * Currently, the only method is "load" which is called in main() to load the user
 * extension.
 */
GeoMOOSE.UX.Extension = OpenLayers.Class({
	initialize: function () {
		/* do nothing on creation */
	},
	load: function() {
		/* do nothing on load */
	},
	CLASS_NAME: 'GeoMOOSE.UX.Extension'
});
